"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shouldProxy = shouldProxy;
exports["default"] = exports.axios = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _https = _interopRequireDefault(require("https"));

var _url = _interopRequireDefault(require("url"));

var _httpsProxyAgent = _interopRequireDefault(require("https-proxy-agent"));

var _i18n = _interopRequireWildcard(require("../i18n"));

var _config = require("./config");

var _login = require("../core/login");

var _helpers = require("./helpers");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var MAX_RETRY_TIMES = 3;

var axios = _axios["default"].create({
  proxy: false
});

exports.axios = axios;
axios.interceptors.request.use(function (config) {
  // i18n.log("request.requestUrl", { url: config.url });
  var proxySettings = getProxySettings(config.url);
  Object.keys(proxySettings).forEach(function (key) {
    config[key] = proxySettings[key];
  });

  if (config.method === 'get' && config.params) {
    var keys = Object.keys(config.params);
    config.url += '?';
    keys.forEach(function (key) {
      config.url += "".concat(key, "=").concat(encodeURIComponent(config.params[key]), "&");
    });
    config.params = {};
  }

  if (config.needAuth) {
    config.headers.hpmAuth = true;
    var cookie = (0, _login.getCachedCookie)();

    if (cookie) {
      var hpmCsrfToken = (0, _login.getCsrfToken)(cookie);
      config.headers.Cookie = cookie;
      config.headers.hpmCsrfToken = hpmCsrfToken;
    }
  }

  return config;
});
createAxiosResponseInterceptor();

function createAxiosResponseInterceptor() {
  var interceptor = axios.interceptors.response.use( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(res) {
      var data, retryTime;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              data = res.data;

              if (!(data.code && data.code !== 200)) {
                _context.next = 12;
                break;
              }

              if (!(data.code === 302)) {
                _context.next = 11;
                break;
              }

              axios.interceptors.response.eject(interceptor);
              _context.next = 6;
              return (0, _login.getTokenLoginCookie)(_helpers.URL.login(), {
                ignoreCache: true
              });

            case 6:
              createAxiosResponseInterceptor();
              retryTime = res.config.retryTime || 0;

              if (!(retryTime < MAX_RETRY_TIMES)) {
                _context.next = 11;
                break;
              }

              res.config.retryTime = retryTime + 1;
              return _context.abrupt("return", axios(res.config));

            case 11:
              throw data;

            case 12:
              return _context.abrupt("return", res);

            case 13:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }(), function (error) {
    throw error;
  });
}

function getProxySettings(visitingUrl) {
  var visitingUrlParsed = _url["default"].parse(visitingUrl);

  var noProxy = process.env.no_proxy || process.env.NO_PROXY;

  if (!shouldProxy(noProxy, visitingUrlParsed.hostname)) {
    return {
      httpsAgent: new _https["default"].Agent({
        rejectUnauthorized: (0, _config.isStrictSsl)()
      })
    };
  }

  if (!(0, _config.isStrictSsl)()) {
    process.removeAllListeners('warning');
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = 0;
  }

  var httpsAgent;
  var httpAgent;
  var httpsProxy = process.env.https_proxy || process.env.HTTPS_PROXY;
  var httpProxy = process.env.http_proxy || process.env.HTTP_PROXY;

  if (httpsProxy) {
    var httpsProxyParsed = parseProxyUrl(decodeURIComponent(httpsProxy));

    var opts = _url["default"].parse(httpsProxyParsed.proxypath);

    if (httpsProxyParsed.auth) {
      opts.auth = httpsProxyParsed.auth;
    }

    httpsAgent = new _httpsProxyAgent["default"](opts);
  }

  if (httpProxy) {
    var httpProxyParsed = parseProxyUrl(decodeURIComponent(httpProxy));

    var _opts = _url["default"].parse(httpProxyParsed.proxypath);

    _opts.protocol = 'http';

    if (httpProxyParsed.auth) {
      _opts.auth = httpProxyParsed.auth;
    }

    httpAgent = new _httpsProxyAgent["default"](_opts);
  }

  return {
    httpsAgent: httpsAgent,
    httpAgent: httpAgent
  };
}

function parseProxyUrl(proxyUrl) {
  var withAuthParsed = /(https:\/\/|http:\/\/)([\s\S]+)@([\s\S]+)/g.exec(proxyUrl);
  var proxypath;
  var auth;

  if (withAuthParsed) {
    if (withAuthParsed.length !== 4) {
      throw (0, _i18n.getI18nMessage)('request.proxyFormat');
    } else {
      auth = withAuthParsed[2];
      proxypath = "".concat(withAuthParsed[1]).concat(withAuthParsed[3]);
    }
  } else {
    proxypath = proxyUrl;
  }

  return {
    proxypath: proxypath,
    auth: auth
  };
}

function shouldProxy(noProxy, hostname) {
  if (!noProxy) {
    return true;
  }

  var noProxyArr = noProxy.split(',').map(function (s) {
    return s.trim();
  });
  var should = !noProxyArr.some(function (proxyElement) {
    if (!proxyElement) {
      return false;
    }

    if (proxyElement === '*') {
      return true;
    }

    if (proxyElement[0] === '.' && hostname.substr(hostname.length - proxyElement.length) === proxyElement) {
      return true;
    }

    if (proxyElement[0] === '*' && hostname.substr(hostname.length - proxyElement.length + 1) === proxyElement.substr(1)) {
      return true;
    }

    return hostname === proxyElement;
  });
  return should;
}

var _default = axios;
exports["default"] = _default;